﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace WindowsFormsApplication1
{
    class _12306Class
    {
        public static CookieCollection Cookies
        {
            get { return cookies; }
        }

        static CookieCollection cookies = new CookieCollection();

        private const string UrlLoginImage = "http://dynamic.12306.cn/otsweb/passCodeAction.do?rand=sjrand";
        private const string UrlMainPage = "https://dynamic.12306.cn/otsweb/main.jsp";
        private const string UrlLoginStep1 = "http://dynamic.12306.cn/otsweb/loginAction.do?method=loginAysnSuggest";
        private const string UrlLoginStep2 = "http://dynamic.12306.cn/otsweb/loginAction.do?method=login";
        private const string UrlFavoriteContacts = "http://dynamic.12306.cn/otsweb/passengerAction.do?method=getPagePassengerAll";

        private const string UrlCreateFavoriteContact1_GetToken = "https://dynamic.12306.cn/otsweb/passengerAction.do?method=initUsualPassenger12306#";
        private const string UrlCreateFavoriteContact2_Create = "https://dynamic.12306.cn/otsweb/passengerAction.do?method=savePassenger";

        private const string UrlGetStations = "https://dynamic.12306.cn/otsweb/js/common/station_name.js";
        private const string UrlGetTrains = "https://dynamic.12306.cn/otsweb/order/querySingleAction.do?method=queryststrainall";


        private const string UrlSelectTrainSeats = "https://dynamic.12306.cn/otsweb/order/querySingleAction.do";


        private const string UrlCreateTask1_PostData = "https://dynamic.12306.cn/otsweb/order/querySingleAction.do?method=submutOrderRequest";
        private const string UrlCreateTask2_Get = "https://dynamic.12306.cn/otsweb/order/confirmPassengerAction.do?method=init";

        /// <summary>
        /// 获取登录图片
        /// </summary>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        /// <returns></returns>
        public static Bitmap GetLoginImage(int? timeout = null, string userAgent = null, CookieCollection cookie = null)
        {
            try
            {
                var response = HttpHelper.CreateGetHttpResponse(UrlLoginImage, timeout, userAgent, cookie ?? Cookies);
                Stream resStream = response.GetResponseStream();//得到验证码数据流
                return new Bitmap(resStream);//初始化Bitmap图片
            }
            catch (Exception)
            {
                return null;
            }
        }

        /// <summary>
        /// 登录首页，获取Cookie
        /// </summary>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        public static void GetMainPage(int? timeout = null, string userAgent = null, CookieCollection cookie = null)
        {
            var response = HttpHelper.CreateGetHttpResponse(UrlMainPage, timeout, userAgent, cookie ?? Cookies, "");

            cookies.Add(response.Cookies);
        }

        /// <summary>
        /// 登录
        /// </summary>
        /// <param name="userName"></param>
        /// <param name="pass"></param>
        /// <param name="verificationCode"></param>
        /// <param name="timeout"></param>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        /// <returns></returns>
        public static LoginResponse Login(string userName, string pass, string verificationCode
            , int? timeout = null, string userAgent = null, CookieCollection cookie = null)
        {
            //   获取登录随机验证码loginRand
            string str = "";
            var response = HttpHelper.CreateGetHttpResponse(UrlLoginStep1, timeout, userAgent, cookie ?? Cookies, "http://dynamic.12306.cn/otsweb/loginAction.do?method=init");
            cookies.Add(response.Cookies);

            try
            {
                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    str = reader.ReadToEnd();
                }

                if (str.Length > 50)
                {
                    return new LoginResponse() { IsLogined = false, Message = "网络异常，请稍后重试。", type = ErrorType.NetworkError };
                }
            }
            catch (Exception)
            {
                return new LoginResponse() { IsLogined = false, Message = "网络可能存在问题，请您重试一下！", type = ErrorType.NetworkError };
            }

            // 解析第一次返回值，获取登录随机验证码loginRand，执行登录
            var v = new IronJS.Hosting.CSharp.Context();
            string strJson = "var json = " + str;
            dynamic dy = v.Execute(strJson);

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("loginUser.user_name", userName);
            dic.Add("nameErrorFocus", "");
            dic.Add("user.password", pass);
            dic.Add("passwordErrorFocus", "");
            dic.Add("randCode", verificationCode);
            dic.Add("randErrorFocus", "");
            dic.Add("loginRand", dy.loginRand);
            response = HttpHelper.CreatePostHttpResponse(UrlLoginStep2, dic, timeout, userAgent, Encoding.UTF8, cookie ?? Cookies, "http://dynamic.12306.cn/otsweb/loginAction.do?method=init");

            cookies.Add(response.Cookies);

            try
            {
                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    str = reader.ReadToEnd();
                }
            }
            catch (Exception)
            {
                return new LoginResponse() { IsLogined = false, Message = "网络可能存在问题，请您重试一下！", type = ErrorType.NetworkError };
            }

            Regex logined = new Regex(@"'[^']*? ，您好！'");

            Regex networkError = new Regex(@"网络可能存在问题，请您重试一下");

            Regex verificationError = new Regex(@"请输入正确的验证码");

            Regex PasswordError = new Regex(@"密码输入错误");

            Regex EmailError = new Regex(@"邮箱不存在!");

            if (logined.IsMatch(str))
            {
                return new LoginResponse() { IsLogined = true, Message = "", type = ErrorType.None, LoginName = logined.Matches(str)[0].Value };
            }
            else if (verificationError.IsMatch(str))
            {
                return new LoginResponse() { IsLogined = false, Message = "请输入正确的验证码！", type = ErrorType.VerificationError };
            }
            else if (networkError.IsMatch(str))
            {
                return new LoginResponse() { IsLogined = false, Message = "网络可能存在问题，请您重试一下！", type = ErrorType.NetworkError };
            }
            else if (PasswordError.IsMatch(str))
            {
                return new LoginResponse() { IsLogined = false, Message = "密码错误！", type = ErrorType.PassordError };
            }
            else if (EmailError.IsMatch(str))
            {
                return new LoginResponse() { IsLogined = false, Message = "邮箱不存在！", type = ErrorType.EmailError };
            }

            return new LoginResponse() { IsLogined = false, Message = "未知错误！", type = ErrorType.OtherError };
        }

        /// <summary>
        /// 获取常用联系人
        /// </summary>
        /// <param name="timeout"></param>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        public static List<Contact> GetFavoriteContacts(int pageIndex, int pageSize, out int count, int? timeout = null, string userAgent = null,
                                               CookieCollection cookie = null)
        {
            List<Contact> contacts = new List<Contact>();
            count = 0;
            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("pageIndex", pageIndex.ToString());
            dic.Add("pageSize", pageSize.ToString());
            dic.Add("passenger_name", "请输入汉字或拼音首字母");
            var response = HttpHelper.CreatePostHttpResponse(UrlFavoriteContacts, dic, timeout, userAgent, Encoding.UTF8, cookie ?? Cookies, "https://dynamic.12306.cn/otsweb/passengerAction.do?method=initUsualPassenger12306");

            cookies.Add(response.Cookies);
            dynamic dy;
            try
            {
                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    var str = reader.ReadToEnd();

                    var v = new IronJS.Hosting.CSharp.Context();
                    string strJson = "var json = " + str;

                    dy = v.Execute(strJson);

                    count = (int)dy.recordCount;

                    for (int i = 0; i < dy.rows.length; i++)
                    {
                        contacts.Add(new Contact()
                        {
                            Name = dy.rows[i].passenger_name,
                            IdTypeName = dy.rows[i].passenger_id_type_name,
                            IdNo = dy.rows[i].passenger_id_no,
                            IdTypeCode = dy.rows[i].passenger_id_type_code,
                            Mobile = dy.rows[i].mobile_no,
                            PassengerTypeName = dy.rows[i].passenger_type_name,
                            PassengerType = dy.rows[i].passenger_type
                        });
                    }
                }
            }
            catch (Exception)
            {

            }

            return contacts;
        }


        /// <summary>
        /// 添加常用联系人，暂时只支持添加成人
        /// </summary>
        /// <param name="name"></param>
        /// <param name="sexCode">男：M，女：F</param>
        /// <param name="bornDate"></param>
        /// <param name="cardType"></param>
        /// <param name="cardNo"></param>
        /// <param name="mobileNo"></param>
        /// <param name="timeout"></param>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        /// <returns></returns>
        public static CreateFavoriteContactResponse CreateCreateFavoriteContact(string name, string sexCode, string bornDate, string cardType, string cardNo, string mobileNo = ""
            , int? timeout = null, string userAgent = null, CookieCollection cookie = null)
        {
            string token = "";
            string str;
            HttpWebResponse response;

            #region 第一步获取Token
            try
            {

                response = HttpHelper.CreateGetHttpResponse(UrlCreateFavoriteContact1_GetToken, timeout, userAgent, cookie ?? Cookies, "");

                cookies.Add(response.Cookies);

                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    str = reader.ReadToEnd();
                }
            }
            catch (Exception)
            {
                return new CreateFavoriteContactResponse() { IsCreated = false, Message = "网络可能存在问题，请您重试一下！", type = ErrorType.NetworkError };
            }


            Regex Token = new Regex("<input type=\"hidden\" name=\"org.apache.struts.taglib.html.TOKEN\" value=\"(?<token>[^\"]*?)\"></div>");

            if (Token.IsMatch(str))
            {
                token = Token.Matches(str)[0].Groups["token"].Value;
            }

            #endregion

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("org.apache.struts.taglib.html.TOKEN", token);
            dic.Add("name", name);
            dic.Add("sex_code", sexCode);
            dic.Add("born_date", bornDate);
            dic.Add("country_code", "CN");
            dic.Add("card_type", cardType);
            dic.Add("card_no", cardNo);
            dic.Add("passenger_type", "1");//暂时只支持添加成人
            dic.Add("mobile_no", mobileNo);
            dic.Add("phone_no", "");
            dic.Add("email", "");
            dic.Add("address", "");
            dic.Add("postalcode", "");
            dic.Add("studentInfo.province_code", "11");
            dic.Add("studentInfo.school_code", "");
            dic.Add("studentInfo.school_name", "简码/汉字");
            dic.Add("studentInfo.department", "");
            dic.Add("studentInfo.school_class", "");
            dic.Add("studentInfo.student_no", "");
            dic.Add("studentInfo.school_system", "4");
            dic.Add("studentInfo.enter_year", "2002");
            dic.Add("studentInfo.preference_card_no", "");
            dic.Add("studentInfo.preference_from_station_name", "简码/汉字");
            dic.Add("studentInfo.preference_from_station_code", "");
            dic.Add("studentInfo.preference_to_station_name", "简码/汉字");
            dic.Add("studentInfo.preference_to_station_code", "");

            try
            {

                response = HttpHelper.CreatePostHttpResponse(UrlCreateFavoriteContact2_Create, dic, timeout, userAgent, Encoding.UTF8, cookie ?? Cookies, "https://dynamic.12306.cn/otsweb/passengerAction.do?method=initAddPassenger&");

                cookies.Add(response.Cookies);

                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    str = reader.ReadToEnd();
                }
            }
            catch (Exception)
            {
                return new CreateFavoriteContactResponse() { IsCreated = false, Message = "网络可能存在问题，请您重试一下！", type = ErrorType.NetworkError };
            }

            Regex message = new Regex("var message = \"(?<message>[^\"]*?)\";");

            if (message.IsMatch(str))
            {
                str = message.Matches(str)[0].Groups["message"].Value;
                if (str.Contains("添加常用联系人成功"))
                {
                    return new CreateFavoriteContactResponse() { IsCreated = true, Message = "", type = ErrorType.None };
                }
                else
                {
                    return new CreateFavoriteContactResponse() { IsCreated = false, Message = str, type = ErrorType.OtherError };
                }
            }

            return new CreateFavoriteContactResponse() { IsCreated = false, Message = "其他错误！", type = ErrorType.OtherError };
        }

        /// <summary>
        /// 获取车站信息
        /// </summary>
        /// <param name="timeout"></param>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        public static List<Station> GetStations(int? timeout = null, string userAgent = null,
                                               CookieCollection cookie = null)
        {
            List<Station> list = new List<Station>();

            var response = HttpHelper.CreateGetHttpResponse(UrlGetStations, timeout, userAgent, cookie ?? Cookies);

            cookies.Add(response.Cookies);
            try
            {
                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    var str = reader.ReadToEnd();

                    Regex stationNamesRegex = new Regex("'(?<stationNames>[^\']*?)'");

                    if (stationNamesRegex.IsMatch(str))
                    {
                        string stationNames = stationNamesRegex.Matches(str)[0].Groups["stationNames"].Value;


                        string[] stations = stationNames.Split('@');

                        foreach (var station in stations)
                        {
                            if (string.IsNullOrEmpty(station))
                            {
                                continue;
                            }

                            string[] names = station.Split('|');

                            list.Add(new Station()
                            {
                                Shorthand = names[0],
                                Name = names[1],
                                Code = names[2],
                                Pinyin = names[3],
                                FirstLetter = names[4],
                                Order = names[5]
                            });
                        }
                    }
                }
            }
            catch (Exception)
            {

            }

            return list;
        }

        /// <summary>
        /// 获取车次信息
        /// </summary>
        /// <param name="data"></param>
        /// <param name="fromstation"></param>
        /// <param name="tostation"></param>
        /// <param name="timeout"></param>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        /// <returns></returns>
        public static List<Train> GetTrains(string data, string fromstation, string tostation,
                                            int? timeout = null, string userAgent = null, CookieCollection cookie = null)
        {
            List<Train> list = new List<Train>();
            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("date", data);
            dic.Add("fromstation", fromstation);
            dic.Add("tostation", tostation);
            dic.Add("starttime", "00:00--24:00");

            string str = "";

            try
            {
                var response = HttpHelper.CreatePostHttpResponse(UrlGetTrains, dic, timeout, userAgent, Encoding.UTF8, cookie ?? Cookies, "https://dynamic.12306.cn/otsweb/passengerAction.do?method=initAddPassenger&");

                cookies.Add(response.Cookies);

                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    str = reader.ReadToEnd();
                }
            }
            catch (Exception)
            {
                return null;
            }

            if (!string.IsNullOrEmpty(str))
            {
                // 解析第一次返回值，获取登录随机验证码loginRand，执行登录
                var v = new IronJS.Hosting.CSharp.Context();
                string strJson = "var json = " + str;
                dynamic dy = v.Execute(strJson);

                for (int i = 0; i < dy.length; i++)
                {
                    list.Add(new Train()
                        {
                            Id = dy[i].id,
                            EndStation = dy[i].end_station_name,
                            EndTime = dy[i].end_time,
                            StartStation = dy[i].start_station_name,
                            StartTime = dy[i].start_time,
                            TrainValue = dy[i].value
                        });
                }
            }

            return list;
        }

        /// <summary>
        /// 获取座位信息
        /// </summary>
        /// <param name="date"></param>
        /// <param name="startStationCode"></param>
        /// <param name="endStationCode"></param>
        /// <param name="trainNo"></param>
        /// <param name="timeout"></param>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        /// <returns></returns>
        public static TrainSeat GetTrainSeats(string date, string startStationCode, string endStationCode, string trainNo,
                                  int? timeout = null, string userAgent = null, CookieCollection cookie = null)
        {
            TrainSeat trainSeat = new TrainSeat();

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("method", "queryLeftTicket");
            dic.Add("orderRequest.train_date", date);
            dic.Add("orderRequest.from_station_telecode", startStationCode);
            dic.Add("orderRequest.to_station_telecode", endStationCode);
            dic.Add("orderRequest.train_no", trainNo);
            dic.Add("trainPassType", "QB");
            dic.Add("trainClass", "QB%23D%23Z%23T%23K%23QT%23");
            dic.Add("includeStudent", "00");
            dic.Add("seatTypeAndNum", "");
            dic.Add("orderRequest.start_time_str", "00%3A00--24%3A00");

            string str = "";

            try
            {
                var response = HttpHelper.CreateGetHttpResponse(UrlSelectTrainSeats + "?" + HttpHelper.CraeteParameter(dic), timeout, userAgent, cookie ?? Cookies, "https://dynamic.12306.cn/otsweb/order/querySingleAction.do?method=init");

                cookies.Add(response.Cookies);

                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    str = reader.ReadToEnd();
                }

                if (str == "-1" || string.IsNullOrEmpty(str))
                {
                    return null;
                }
            }
            catch (Exception)
            {
                return null;
            }

            string[] strs = str.Split(',');


            for (int i = 5; i < 15; i++)
            {
                if (i == 9)
                {
                    continue;
                }

                int m;

                if (int.TryParse(strs[i], out m))
                {
                    trainSeat.List.Add(Seat.Create((SeatsType)i, m.ToString()));
                }
                else if (strs[i].Contains("有"))
                {
                    trainSeat.List.Add(Seat.Create((SeatsType)i, "有"));
                }
                else if (strs[i].Contains("无"))
                {
                    trainSeat.List.Add(Seat.Create((SeatsType)i, "无"));
                }
            }



            Regex stationNamesRegex = new Regex("\\(\'(?<body>[^\']*?)\'\\)");

            if (stationNamesRegex.IsMatch(strs[strs.Count() - 1]))
            {
                string stationNames = stationNamesRegex.Matches(strs[strs.Count() - 1])[0].Groups["body"].Value;

                string[] msgs = stationNames.Split('#');

                trainSeat.station_train_code = msgs[0];
                trainSeat.train_date = date;
                trainSeat.seattype_num = "";
                trainSeat.from_station_telecode = msgs[4];
                trainSeat.to_station_telecode = msgs[5];
                trainSeat.include_student = "00";
                trainSeat.from_station_telecode_name = msgs[7];
                trainSeat.to_station_telecode_name = msgs[8];
                trainSeat.round_train_date = DateTime.Now.ToString("yyyy-MM-dd");
                trainSeat.round_start_time_str = "00:00--24:00";
                trainSeat.single_round_type = "1";
                trainSeat.train_pass_type = "QB";
                trainSeat.train_class_arr = "QB#D#Z#T#K#QT#";
                trainSeat.start_time_str = "00:00--24:00";
                trainSeat.lishi = msgs[1];
                trainSeat.train_start_time = msgs[2];
                trainSeat.trainno4 = msgs[3];
                trainSeat.arrive_time = msgs[6];
                trainSeat.from_station_name = msgs[7];
                trainSeat.to_station_name = msgs[8];
                trainSeat.from_station_no = msgs[9];
                trainSeat.to_station_no = msgs[10];
                trainSeat.ypInfoDetail = msgs[11];
                trainSeat.mmStr = msgs[12];
                trainSeat.locationCode = msgs[13];
            }

            return trainSeat;
        }

        /// <summary>
        /// 获取座位信息
        /// </summary>
        /// <param name="date"></param>
        /// <param name="startStationCode"></param>
        /// <param name="endStationCode"></param>
        /// <param name="trainNo"></param>
        /// <param name="timeout"></param>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        /// <returns></returns>
        public static TrainSeat CreateTask(TrainSeat trainSeat,
                                  int? timeout = null, string userAgent = null, CookieCollection cookie = null)
        {
            Dictionary<string, string> dic = new Dictionary<string, string>();

            dic.Add("station_train_code", trainSeat.station_train_code);
            dic.Add("train_date", trainSeat.train_date);
            dic.Add("seattype_num", trainSeat.seattype_num);
            dic.Add("from_station_telecode", trainSeat.from_station_telecode);
            dic.Add("to_station_telecode", trainSeat.to_station_telecode);
            dic.Add("include_student", trainSeat.include_student);
            dic.Add("from_station_telecode_name", trainSeat.from_station_telecode_name);
            dic.Add("to_station_telecode_name", trainSeat.to_station_telecode_name);
            dic.Add("round_train_date", trainSeat.round_train_date);
            dic.Add("round_start_time_str", trainSeat.round_start_time_str);
            dic.Add("single_round_type", trainSeat.single_round_type);
            dic.Add("train_pass_type", trainSeat.train_pass_type);
            dic.Add("train_class_arr", trainSeat.train_class_arr);
            dic.Add("start_time_str", trainSeat.start_time_str);
            dic.Add("lishi", trainSeat.lishi);
            dic.Add("train_start_time", trainSeat.train_start_time);
            dic.Add("trainno4", trainSeat.trainno4);
            dic.Add("arrive_time", trainSeat.arrive_time);
            dic.Add("from_station_name", trainSeat.from_station_name);
            dic.Add("to_station_name", trainSeat.to_station_name);
            dic.Add("from_station_no", trainSeat.from_station_no);
            dic.Add("to_station_no", trainSeat.to_station_no);
            dic.Add("ypInfoDetail", trainSeat.ypInfoDetail);
            dic.Add("mmStr", trainSeat.mmStr);
            dic.Add("locationCode", trainSeat.locationCode);

            string str = "";

            try
            {
                var response = HttpHelper.CreatePostHttpResponse(UrlCreateTask1_PostData, dic, timeout, userAgent, UTF8Encoding.UTF8, cookie ?? Cookies
                                                                    , "https://dynamic.12306.cn/otsweb/order/querySingleAction.do?method=init");

                cookies.Add(response.Cookies);

                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    str = reader.ReadToEnd();
                }
            }
            catch (Exception)
            {
                return null;
            }

            try
            {
                var response = HttpHelper.CreateGetHttpResponse(UrlCreateTask2_Get, timeout, userAgent, cookie ?? Cookies
                                                                    , "https://dynamic.12306.cn/otsweb/order/querySingleAction.do?method=init");

                cookies.Add(response.Cookies);

                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    str = reader.ReadToEnd();
                }
            }
            catch (Exception)
            {
                return null;
            }

            return null;
            // return trainSeat;
        }
    }



    class Contact
    {
        public string Name;
        public string IdTypeName;
        public string IdNo;
        public string IdTypeCode;
        public string Mobile;
        public string PassengerTypeName;
        public string PassengerType;

        public override string ToString()
        {
            return string.Format("{0}\t{1}\t{2}", Name, IdTypeName, IdNo);
        }
    }

    class LoginResponse
    {
        public bool IsLogined;
        public string Message;
        public string LoginName;
        public ErrorType type;
    }

    class CreateFavoriteContactResponse
    {
        public bool IsCreated;
        public string Message;
        public ErrorType type;
    }

    public enum ErrorType
    {
        None,
        NetworkError,
        VerificationError,
        PassordError,
        EmailError,
        OtherError
    }
}
